@@ -1,9 +1,13 @@
 # -*- coding: utf-8 -*-
 import logging
+from copy import copy
+
 import pika
+from django.views.debug import ExceptionReporter
 from pika import credentials
-from .formatters import JSONFormatter
+
 from .filters import FieldFilter
+from .formatters import JSONFormatter
 
 
 class RabbitMQHandler(logging.Handler):
@@ -11,32 +15,35 @@ class RabbitMQHandler(logging.Handler):
     Python/Django logging handler to ship logs to RabbitMQ.
     Inspired by: https://github.com/ziXiong/MQHandler
     """
-
-    def __init__(self, level=logging.NOTSET, formatter=JSONFormatter(),
+    def __init__(self, level=logging.NOTSET, formatter=None,
                  host='localhost', port=5672, connection_params=None,
                  username=None, password=None,
                  exchange='log', declare_exchange=False,
-                 routing_key_format="{name}.{level}", close_after_emit=False,
-                 fields=None, fields_under_root=True, message_headers=None):
-        """
-        Initialize the handler.
-
-        :param level:              Logs level.
-        :param formatter:          Use custom formatter for the logs.
-        :param host:               RabbitMQ host. Default localhost
-        :param port:               RabbitMQ Port. Default 5672
-        :param connection_params:  Allow extra params to connect with RabbitMQ.
-        :param message_headers:    A dictionary of headers to be published with the message. Optional.
-        :param username:           Username in case of authentication.
-        :param password:           Password for the username.
-        :param exchange:           Send logs using this exchange.
-        :param declare_exchange:   Whether or not to declare the exchange.
-        :param routing_key_format: Customize how messages will be routed to the queues.
-        :param close_after_emit:   Close connection after emit the record?
-        :param fields:             Send these fields as part of all logs.
-        :param fields_under_root:  Merge the fields in the root object.
-        """
-
+                 routing_key_format="{name}.{level}",
+                 routing_key_formatter=None,
+                 close_after_emit=False,
+                 fields=None, fields_under_root=True, message_headers=None,
+                 record_fields=None, exclude_record_fields=None):
+        # Initialize the handler.
+        #
+        # :param level:                 Logs level.
+        # :param formatter:             Use custom formatter for the logs.
+        # :param host:                  RabbitMQ host. Default localhost
+        # :param port:                  RabbitMQ Port. Default 5672
+        # :param connection_params:     Allow extra params to connect with RabbitMQ.
+        # :param message_headers:       A dictionary of headers to be published with the message. Optional.
+        # :param username:              Username in case of authentication.
+        # :param password:              Password for the username.
+        # :param exchange:              Send logs using this exchange.
+        # :param declare_exchange:      Whether or not to declare the exchange.
+        # :param routing_key_format:    Customize how messages will be routed to the queues.
+        # :param routing_key_formatter: Override how messages will be routed to the queues.
+        #                               Formatter is passed record object.
+        # :param close_after_emit:      Close connection after emit the record?
+        # :param fields:                Send these fields as part of all logs.
+        # :param fields_under_root:     Merge the fields in the root object.
+        # :record_fields                A set of attributes that should be preserved from the record object.
+        # :exclude_record_fields        A set of attributes that should be ignored from the record object.
         super(RabbitMQHandler, self).__init__(level=level)
 
         # Important instances/properties.
@@ -60,8 +67,14 @@ class RabbitMQHandler(logging.Handler):
         # Extra params for message publication
         self.message_headers = message_headers
 
+        # Save routing-key formatter.
+        self.routing_key_formatter = routing_key_formatter
+
         # Logging.
-        self.formatter = formatter
+        self.formatter = formatter or JSONFormatter(
+            include=record_fields,
+            exclude=exclude_record_fields
+        )
         self.fields = fields if isinstance(fields, dict) else {}
         self.fields_under_root = fields_under_root
 
@@ -75,7 +88,6 @@ class RabbitMQHandler(logging.Handler):
         """
         Connect to RabbitMQ.
         """
-
         # Set logger for pika.
         # See if something went wrong connecting to RabbitMQ.
         handler = logging.StreamHandler()
@@ -92,7 +104,7 @@ class RabbitMQHandler(logging.Handler):
             self.channel = self.connection.channel()
 
         if self.exchange_declared is False:
-            self.channel.exchange_declare(exchange=self.exchange, type='topic', durable=True, auto_delete=False)
+            self.channel.exchange_declare(exchange=self.exchange, exchange_type='topic', durable=True, auto_delete=False)
             self.exchange_declared = True
 
         # Manually remove logger to avoid shutdown message.
@@ -102,7 +114,6 @@ class RabbitMQHandler(logging.Handler):
         """
         Close active connection.
         """
-
         if self.channel:
             self.channel.close()
 
@@ -118,12 +129,35 @@ class RabbitMQHandler(logging.Handler):
             if not self.connection or self.connection.is_closed or not self.channel or self.channel.is_closed:
                 self.open_connection()
 
-            routing_key = self.routing_key_format.format(name=record.name, level=record.levelname)
+            if self.routing_key_formatter:
+                routing_key = self.routing_key_formatter(record)
+            else:
+                routing_key = self.routing_key_format.format(
+                    name=record.name,
+                    level=record.levelname
+                )
+
+            if hasattr(record, 'request'):
+                no_exc_record = copy(record)
+                del no_exc_record.exc_info
+                del no_exc_record.exc_text
+                del no_exc_record.request
+
+                if record.exc_info:
+                    exc_info = record.exc_info
+                else:
+                    exc_info = (None, record.getMessage(), None)
+
+                reporter = ExceptionReporter(record.request, is_email=False, *exc_info)
+                no_exc_record.traceback = reporter.get_traceback_text()
+                formatted = self.format(no_exc_record)
+            else:
+                formatted = self.format(record)
 
             self.channel.basic_publish(
                 exchange=self.exchange,
                 routing_key=routing_key,
-                body=self.format(record),
+                body=formatted,
                 properties=pika.BasicProperties(
                     delivery_mode=2,
                     headers=self.message_headers
@@ -143,7 +177,6 @@ class RabbitMQHandler(logging.Handler):
         """
         Free resources.
         """
-
         self.acquire()
 
         try:
@@ -155,5 +188,4 @@ class RabbitMQHandler(logging.Handler):
         """
         Close when destroy de instance.
         """
-
         self.close()
